/*
 * utrace infrastructure interface for debugging user processes
 *
 * Copyright (C) 2006, 2007, 2008 Red Hat, Inc.  All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2.
 *
 * Red Hat Author: Roland McGrath.
 *
 * This interface allows for notification of interesting events in a thread.
 * It also mediates access to thread state such as registers.
 * Multiple unrelated users can be associated with a single thread.
 * We call each of these a tracing engine.
 *
 * A tracing engine starts by calling utrace_attach() on the chosen thread,
 * passing in a set of hooks (&struct utrace_engine_ops), and some
 * associated data.  This produces a &struct utrace_attached_engine, which
 * is the handle used for all other operations.  An attached engine has its
 * ops vector, its data, and a flags word controlled by utrace_set_flags().
 *
 * Each engine's flags word contains two kinds of flags: events of
 * interest, and action state flags.
 *
 * For each event flag that is set, that engine will get the
 * appropriate ops->report_*() callback when the event occurs.  The
 * &struct utrace_engine_ops need not provide callbacks for an event
 * unless the engine sets one of the associated event flags.
 *
 * Action state flags change the normal behavior of the thread.
 * These bits are in %UTRACE_ACTION_STATE_MASK; these can be OR'd into
 * flags set with utrace_set_flags().  Also, every callback that return
 * an action value can reset these bits for the engine (see below).
 *
 * The bits %UTRACE_ACTION_STATE_MASK of all attached engines are OR'd
 * together, so each action is in force as long as any engine requests it.
 * As long as some engine sets the %UTRACE_ACTION_QUIESCE flag, the thread
 * will block and not resume running user code.  When the last engine
 * clears its %UTRACE_ACTION_QUIESCE flag, the thread will resume running.
 */

#ifndef _LINUX_UTRACE_H
#define _LINUX_UTRACE_H	1

#include <linux/list.h>
#include <linux/rcupdate.h>
#include <linux/signal.h>
#include <linux/sched.h>

struct linux_binprm;
struct pt_regs;
struct utrace;
struct utrace_engine_ops;
struct utrace_attached_engine;
struct utrace_examiner;
struct user_regset;
struct user_regset_view;
enum utrace_resume_action;

/*
 * Event bits in &struct task_struct.utrace_flags and
 * &struct utrace_attached_engine.flags.
 */
enum utrace_events {
	_UTRACE_EVENT_QUIESCE,	/* Thread is available for examination.  */
	_UTRACE_EVENT_REAP,  	/* Zombie reaped, no more tracing possible.  */
	_UTRACE_EVENT_CLONE,	/* Successful clone/fork/vfork just done.  */
	_UTRACE_EVENT_EXEC,	/* Successful execve just completed.  */
	_UTRACE_EVENT_EXIT,	/* Thread exit in progress.  */
	_UTRACE_EVENT_DEATH,	/* Thread has died.  */
	_UTRACE_EVENT_SYSCALL_ENTRY, /* User entered kernel for system call. */
	_UTRACE_EVENT_SYSCALL_EXIT, /* Returning to user after system call.  */
	_UTRACE_EVENT_SIGNAL,	/* Signal delivery will run a user handler.  */
	_UTRACE_EVENT_SIGNAL_IGN, /* No-op signal to be delivered.  */
	_UTRACE_EVENT_SIGNAL_STOP, /* Signal delivery will suspend.  */
	_UTRACE_EVENT_SIGNAL_TERM, /* Signal delivery will terminate.  */
	_UTRACE_EVENT_SIGNAL_CORE, /* Signal delivery will dump core.  */
	_UTRACE_EVENT_JCTL,	/* Job control stop or continue completed.  */
	_UTRACE_NEVENTS
};
#define UTRACE_EVENT(type)	(1UL << _UTRACE_EVENT_##type)

/*
 * All the kinds of signal events.  These all use the report_signal callback.
 */
#define UTRACE_EVENT_SIGNAL_ALL	(UTRACE_EVENT(SIGNAL) \
				 | UTRACE_EVENT(SIGNAL_IGN) \
				 | UTRACE_EVENT(SIGNAL_STOP) \
				 | UTRACE_EVENT(SIGNAL_TERM) \
				 | UTRACE_EVENT(SIGNAL_CORE))
/*
 * Both kinds of syscall events; these call the report_syscall_entry and
 * report_syscall_exit callbacks, respectively.
 */
#define UTRACE_EVENT_SYSCALL	\
	(UTRACE_EVENT(SYSCALL_ENTRY) | UTRACE_EVENT(SYSCALL_EXIT))

/*
 * Hooks in <linux/tracehook.h> call these entry points to the
 * utrace dispatch.  They are weak references here only so
 * tracehook.h doesn't need to #ifndef CONFIG_UTRACE them to
 * avoid external references in case of unoptimized compilation
 */
void utrace_release_task(struct task_struct *)
	__attribute__((weak));
bool utrace_interrupt_pending(void)
	__attribute__((weak));
void utrace_resume(struct task_struct *, struct pt_regs *)
	__attribute__((weak));
int utrace_get_signal(struct task_struct *, struct pt_regs *,
		      siginfo_t *, struct k_sigaction *)
	__attribute__((weak));
void utrace_report_clone(unsigned long, struct task_struct *)
	__attribute__((weak));
void utrace_report_exit(long *exit_code)
	__attribute__((weak));
void utrace_report_death(struct task_struct *, struct utrace *, bool, int)
	__attribute__((weak));
void utrace_report_jctl(int notify, int type)
	__attribute__((weak));
void utrace_report_exec(struct linux_binfmt *, struct linux_binprm *,
			struct pt_regs *regs)
	__attribute__((weak));
bool utrace_report_syscall_entry(struct pt_regs *)
	__attribute__((weak));
void utrace_report_syscall_exit(struct pt_regs *)
	__attribute__((weak));
struct task_struct *utrace_tracer_task(struct task_struct *)
	__attribute__((weak));
int utrace_unsafe_exec(struct task_struct *)
	__attribute__((weak));
void utrace_signal_handler(struct task_struct *, int)
	__attribute__((weak));

#ifndef CONFIG_UTRACE

/*
 * <linux/tracehook.h> uses these accessors to avoid #ifdef CONFIG_UTRACE.
 */
static unsigned long task_utrace_flags(struct task_struct *task)
{
	return 0;
}
static struct utrace *task_utrace_struct(struct task_struct *task)
{
	return NULL;
}
static inline void utrace_init_task(struct task_struct *child)
{
}

static inline void task_utrace_proc_status(struct seq_file *m,
					   struct task_struct *p)
{
}

#else  /* CONFIG_UTRACE */

static inline unsigned long task_utrace_flags(struct task_struct *task)
{
	return task->utrace_flags;
}

static inline struct utrace *task_utrace_struct(struct task_struct *task)
{
	return task->utrace;
}

static inline void utrace_init_task(struct task_struct *child)
{
	child->utrace_flags = 0;
	child->utrace = NULL;
}

void task_utrace_proc_status(struct seq_file *m, struct task_struct *p);

/*
 * These are the exported entry points for tracing engines to use.
 * See kernel/utrace.c for their kerneldoc comments with interface details.
 */
struct utrace_attached_engine *utrace_attach(struct task_struct *, int flags,
					     const struct utrace_engine_ops *,
					     void *data);
int utrace_control(struct task_struct *, struct utrace_attached_engine *,
		   enum utrace_resume_action);
int utrace_set_events(struct task_struct *, struct utrace_attached_engine *,
		      unsigned long eventmask);
int utrace_prepare_examine(struct task_struct *,
			   struct utrace_attached_engine *,
			   struct utrace_examiner *);
int utrace_finish_examine(struct task_struct *,
			  struct utrace_attached_engine *,
			  struct utrace_examiner *);

/**
 * enum utrace_resume_action - engine's choice of action for a traced task
 * @UTRACE_STOP:		Stay quiescent after callbacks.
 * @UTRACE_REPORT:		Make some callback soon.
 * @UTRACE_INTERRUPT:		Make report_signal callback soon.
 * @UTRACE_SINGLESTEP:		Resume in user mode for one instruction.
 * @UTRACE_BLOCKSTEP:		Resume in user mode until next branch.
 * @UTRACE_RESUME:		Resume normally in user mode.
 * @UTRACE_DETACH:		Detach my engine (implies %UTRACE_RESUME).
 *
 * See utrace_control() for detailed descriptions of each action.  This is
 * encoded in the @action argument and the return value for every callback
 * with a &u32 return value.
 *
 * The order of these is important.  When there is more than one engine,
 * each supplies its choice and the smallest value prevails.
 */
enum utrace_resume_action {
	UTRACE_STOP,
	UTRACE_REPORT,
	UTRACE_INTERRUPT,
	UTRACE_SINGLESTEP,
	UTRACE_BLOCKSTEP,
	UTRACE_RESUME,
	UTRACE_DETACH
};
#define	UTRACE_RESUME_MASK	0x0f

/**
 * utrace_resume_action - &enum utrace_resume_action from callback action
 * @action:		&u32 callback @action argument or return value
 *
 * This extracts the &enum utrace_resume_action from @action,
 * which is the @action argument to a &struct utrace_engine_ops
 * callback or the return value from one.
 */
static inline enum utrace_resume_action utrace_resume_action(u32 action)
{
	return action & UTRACE_RESUME_MASK;
}

/**
 * enum utrace_signal_action - disposition of signal
 * @UTRACE_SIGNAL_DELIVER:	Deliver according to sigaction.
 * @UTRACE_SIGNAL_IGN:		Ignore the signal.
 * @UTRACE_SIGNAL_TERM:		Terminate the process.
 * @UTRACE_SIGNAL_CORE:		Terminate with core dump.
 * @UTRACE_SIGNAL_STOP:		Deliver as absolute stop.
 * @UTRACE_SIGNAL_TSTP:		Deliver as job control stop.
 * @UTRACE_SIGNAL_REPORT:	Reporting before pending signals.
 * @UTRACE_SIGNAL_HANDLER:	Report after signal handler setup.
 *
 * This is encoded in the @action argument and the return value for
 * a @report_signal callback.  It says what will happen to the
 * signal described by the &siginfo_t parameter to the callback.
 *
 * The %UTRACE_SIGNAL_REPORT value is used in an @action argument when
 * a tracing report is being made before dequeuing any pending signal.
 * If this is immediately after a signal handler has been set up, then
 * %UTRACE_SIGNAL_HANDLER is used instead.  A @report_signal callback
 * that uses %UTRACE_SIGNAL_DELIVER | %UTRACE_SINGLESTEP will ensure
 * it sees a %UTRACE_SIGNAL_HANDLER report.
 */
enum utrace_signal_action {
	UTRACE_SIGNAL_DELIVER	= 0x00,
	UTRACE_SIGNAL_IGN	= 0x10,
	UTRACE_SIGNAL_TERM	= 0x20,
	UTRACE_SIGNAL_CORE	= 0x30,
	UTRACE_SIGNAL_STOP	= 0x40,
	UTRACE_SIGNAL_TSTP	= 0x50,
	UTRACE_SIGNAL_REPORT	= 0x60,
	UTRACE_SIGNAL_HANDLER	= 0x70
};
#define	UTRACE_SIGNAL_MASK	0xf0

/**
 * utrace_signal_action - &enum utrace_signal_action from callback action
 * @action:		@report_signal callback @action argument or return value
 *
 * This extracts the &enum utrace_signal_action from @action, which
 * is the @action argument to a @report_signal callback or the
 * return value from one.
 */
static inline enum utrace_signal_action utrace_signal_action(u32 action)
{
	return action & UTRACE_SIGNAL_MASK;
}

#define UTRACE_SIGNAL_HOLD	0x100 /* Flag, push signal back on queue.  */

/**
 * enum utrace_syscall_action - disposition of signal
 * @UTRACE_SYSCALL_RUN:		Run the system call.
 * @UTRACE_SYSCALL_ABORT:	Don't run the system call.
 *
 * This is encoded in the @action argument and the return value for
 * a @report_syscall_entry callback.
 */
enum utrace_syscall_action {
	UTRACE_SYSCALL_RUN	= 0x00,
	UTRACE_SYSCALL_ABORT	= 0x10
};
#define	UTRACE_SYSCALL_MASK	0xf0

/**
 * utrace_syscall_action - &enum utrace_syscall_action from callback action
 * @action:		@report_syscall_entry callback @action or return value
 *
 * This extracts the &enum utrace_syscall_action from @action, which
 * is the @action argument to a @report_syscall_entry callback or the
 * return value from one.
 */
static inline enum utrace_syscall_action utrace_syscall_action(u32 action)
{
	return action & UTRACE_SYSCALL_MASK;
}


#define	UTRACE_HIDE_EVENT	XXX

/*
 * Flags for utrace_attach().
 */
#define UTRACE_ATTACH_CREATE		0x0010 /* Attach a new engine.  */
#define UTRACE_ATTACH_EXCLUSIVE		0x0020 /* Refuse if existing match.  */
#define UTRACE_ATTACH_MATCH_OPS		0x0001 /* Match engines on ops.  */
#define UTRACE_ATTACH_MATCH_DATA	0x0002 /* Match engines on data.  */
#define UTRACE_ATTACH_MATCH_MASK	0x000f

/**
 * struct utrace_attached_engine - Per-engine per-thread structure.
 * @ops: &struct utrace_engine_ops pointer passed to utrace_attach()
 * @data: engine-private void * passed to utrace_attach()
 * @flags: current flags set by utrace_set_flags()
 *
 * The task itself never has to worry about engines detaching while
 * it's doing event callbacks.  These structures are freed only when
 * the task is quiescent.  For other parties, the list is protected
 * by RCU and utrace->lock.
 */
struct utrace_attached_engine {
/* private: */
	struct list_head entry;	/* Entry on thread's utrace.engines list.  */
	struct rcu_head rhead;
	atomic_t check_dead;

/* public: */
	const struct utrace_engine_ops *ops;
	void *data;

	unsigned long flags;
};

struct utrace_engine_ops {
	/*
	 * Event reporting hooks.
	 *
	 * Return values contain %UTRACE_ACTION_* flag bits.
	 * The %UTRACE_ACTION_OP_MASK bits are specific to each kind of event.
	 *
	 * All report_*() hooks are called with no locks held, in a generally
	 * safe environment when we will be returning to user mode soon.
	 * It is fine to block for memory allocation and the like, but all
	 * hooks are *asynchronous* and must not block on external events.
	 * If you want the thread to block, request %UTRACE_ACTION_QUIESCE in
	 * your hook; then later wake it up with utrace_set_flags().
	 */

	/*
	 * Event reported for parent, before child might run.
	 * The %PF_STARTING flag prevents other engines from attaching
	 * before this one has its chance.
	 */
	u32 (*report_clone)(enum utrace_resume_action action,
			    struct utrace_attached_engine *engine,
			    struct task_struct *parent,
			    unsigned long clone_flags,
			    struct task_struct *child);

	/*
	 * Event reported after %UTRACE_ACTION_QUIESCE is set, when the target
	 * thread is quiescent.  Either it's the current thread, or it's in
	 * %TASK_TRACED or %TASK_STOPPED and will not resume running until the
	 * %UTRACE_ACTION_QUIESCE flag is no longer asserted by any engine.
	 */
	u32 (*report_quiesce)(enum utrace_resume_action action,
			      struct utrace_attached_engine *engine,
			      struct task_struct *task,
			      unsigned long event);

	/*
	 * Thread dequeuing a signal to be delivered.
	 * The @action and @return_ka values say what %UTRACE_ACTION_RESUME
	 * will do (possibly already influenced by another tracing engine).
	 * An %UTRACE_SIGNAL_* return value overrides the signal disposition.
	 * The @info data (including @info->si_signo) can be changed at will.
	 * Changing @return_ka affects the sigaction that will be used.
	 * The @orig_ka value is the one in force before other tracing
	 * engines intervened.
	 */
	u32 (*report_signal)(u32 action,
			     struct utrace_attached_engine *engine,
			     struct task_struct *task,
			     struct pt_regs *regs,
			     siginfo_t *info,
			     const struct k_sigaction *orig_ka,
			     struct k_sigaction *return_ka);

	/*
	 * Job control event; @type is %CLD_STOPPED or %CLD_CONTINUED.
	 * If @notify is true, @task is the last thread to stop
	 * and so will send %SIGCHLD to its parent after this callback.
	 */
	u32 (*report_jctl)(enum utrace_resume_action action,
			   struct utrace_attached_engine *engine,
			   struct task_struct *task,
			   bool notify, int type);

	/*
	 * Thread has just completed an exec.
	 * The initial user register state is handy to be tweaked directly.
	 */
	u32 (*report_exec)(enum utrace_resume_action action,
			   struct utrace_attached_engine *engine,
			   struct task_struct *task,
			   const struct linux_binfmt *fmt,
			   const struct linux_binprm *bprm,
			   struct pt_regs *regs);

	/*
	 * Thread has entered the kernel to request a system call.
	 * The user register state is handy to be tweaked directly.
	 * The @action argument contains an &enum utrace_syscall_action,
	 * use utrace_syscall_action() to extract it.  The return value
	 * overrides the last engine's action for the system call.
	 * If the final action is %UTRACE_SYSCALL_ABORT, no system call
	 * is made.
	 */
	u32 (*report_syscall_entry)(u32 action,
				    struct utrace_attached_engine *engine,
				    struct task_struct *task,
				    struct pt_regs *regs);

	/*
	 * Thread is about to leave the kernel after a system call request.
	 * The user register state is handy to be tweaked directly.
	 */
	u32 (*report_syscall_exit)(enum utrace_resume_action action,
				   struct utrace_attached_engine *engine,
				   struct task_struct *task,
				   struct pt_regs *regs);

	/*
	 * Thread is exiting and cannot be prevented from doing so,
	 * but all its state is still live.  The @code value will be
	 * the wait result seen by the parent, and can be changed by
	 * this engine or others.  The @orig_code value is the real
	 * status, not changed by any tracing engine.
	 */
	u32 (*report_exit)(enum utrace_resume_action action,
			   struct utrace_attached_engine *engine,
			   struct task_struct *task,
			   long orig_code, long *code);

	/*
	 * Thread is really dead now.  It might be reaped by its parent at
	 * any time, or self-reap immediately.  Though the actual reaping
	 * may happen in parallel, a report_reap() callback will always be
	 * ordered after a report_death() callback.
	 */
	u32 (*report_death)(struct utrace_attached_engine *engine,
			    struct task_struct *task,
			    bool group_dead, int signal);

	/*
	 * Called when someone reaps the dead task (parent, init, or self).
	 * No more callbacks are made after this one.
	 * The engine is always detached.
	 * There is nothing more a tracing engine can do about this thread.
	 */
	void (*report_reap)(struct utrace_attached_engine *engine,
			    struct task_struct *task);

	/*
	 * Miscellaneous hooks.  These are not associated with event reports.
	 * Any of these may be null if the engine has nothing to say.
	 * These hooks are called in more constrained environments and should
	 * not block or do very much.
	 */

	/*
	 * Return %LSM_UNSAFE_* bits that apply to the exec in progress
	 * due to tracing done by this engine.  These bits indicate that
	 * someone is able to examine the process and so a set-UID or similar
	 * privilege escalation may not be safe to permit.
	 *
	 * Called with task_lock() held.
	 */
	int (*unsafe_exec)(struct utrace_attached_engine *engine,
			   struct task_struct *target);

	/*
	 * Return the &struct task_struct for the task using ptrace on this
	 * one, or %NULL.  Always called with rcu_read_lock() held to keep the
	 * returned struct alive.
	 *
	 * At exec time, this may be called with task_lock(target) still
	 * held from when unsafe_exec() was just called.  In that case it
	 * must give results consistent with those unsafe_exec() results,
	 * i.e. non-%NULL if any %LSM_UNSAFE_PTRACE_* bits were set.
	 *
	 * The value is also used to display after "TracerPid:" in
	 * /proc/PID/status, where it is called with only rcu_read_lock held.
	 *
	 * If this engine returns %NULL, another engine may supply the result.
	 */
	struct task_struct *(*tracer_task)(struct utrace_attached_engine *,
					   struct task_struct *target);
};

/**
 * struct utrace_examiner - private state for using utrace_prepare_examine()
 * @dummy:		all fields are private, none described here
 *
 * The members of &struct utrace_examiner are private to the implementation.
 * This data type holds the state from a call to utrace_prepare_examine()
 * to be used by a call to utrace_finish_examine().
 */
struct utrace_examiner {
/* private: */
	long state;	    /* cache of task_struct.state */
	unsigned long ncsw; /* cache of wait_task_inactive() return value */
/* public: */
	struct {} dummy;
};

#endif	/* CONFIG_UTRACE */

#endif	/* linux/utrace.h */
